@@ -149,6 +149,7 @@ typedef struct {
 	GList          *dirs;
 
 	char           *catalog_path;
+	gboolean        search_comments;
 } DialogData;
 
 
@@ -1108,9 +1109,65 @@ add_file_list (DialogData *data, GList *file_list)
 }
 
 
+static char *
+uri_from_comment_uri (const char *comment_uri,
+		      gboolean    is_file)
+{
+	char *base;
+	char *uri;
+	int   comment_uri_l;
+	int   base_l;
+
+	if (comment_uri == NULL)
+		return NULL;
+
+	comment_uri_l = strlen (comment_uri);
+
+	base = comments_get_comment_dir (NULL);
+	base_l = strlen (base);
+
+	if (comment_uri_l == base_l) 
+		uri = g_strdup ("/");
+
+	else if (comment_uri_l > base_l) {
+		uri = g_strdup (comment_uri + base_l);
+
+		if (is_file) { /* remove the comment extension. */
+			int uri_l = comment_uri_l - base_l;
+			uri [uri_l - strlen (COMMENT_EXT)] = 0;
+		}
+
+	} else
+		uri = NULL;
+	
+	g_free (base);
+
+	return uri;
+}
+
+
 static void search_dir_async (DialogData *data, gchar *dir);
 
 
+static gboolean
+cache_dir (const char *folder)
+{
+	if (folder == NULL)
+		return FALSE;
+
+	if (strcmp (folder, ".nautilus") == 0)
+		return TRUE;
+
+	if (strcmp (folder, ".thumbnails") == 0)
+		return TRUE;
+
+	if (strcmp (folder, ".xvpics") == 0)
+		return TRUE;
+
+	return FALSE;
+}
+
+
 static void
 directory_load_cb (GnomeVFSAsyncHandle *handle,
 		   GnomeVFSResult       result,
@@ -1124,7 +1181,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 
 	for (node = list; node != NULL; node = node->next) {
 		GnomeVFSURI *full_uri = NULL;
-		gchar       *str_uri, *unesc_uri;
+		char        *str_uri, *unesc_uri;
 
 		info = node->data;
 
@@ -1134,6 +1191,13 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, NULL);
 
+			if (data->search_comments) {
+				char *tmp;
+				tmp = uri_from_comment_uri (unesc_uri, TRUE);
+				g_free (unesc_uri);
+				unesc_uri = tmp;
+			}
+
 			if (file_respects_search_criteria (data, unesc_uri)) 
 				files = g_list_prepend (files, unesc_uri);
 			else
@@ -1150,11 +1214,19 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 			str_uri = gnome_vfs_uri_to_string (full_uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 			unesc_uri = gnome_vfs_unescape_string (str_uri, NULL);
 
+			if (data->search_comments) {
+				char *tmp;
+				tmp = uri_from_comment_uri (unesc_uri, FALSE);
+				g_free (unesc_uri);
+				unesc_uri = tmp;
+			}
+
 			data->dirs = g_list_prepend (data->dirs,  unesc_uri);
 			g_free (str_uri);
 			break;
 
 		default:
+			break;
 		}
 
 		if (full_uri)
@@ -1174,7 +1246,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 		
 		do {
 			GList *first_dir;
-			gchar *dir;
+			char  *dir;
 
 			if (data->dirs == NULL) {
 				search_finished (data);
@@ -1183,21 +1255,21 @@ directory_load_cb (GnomeVFSAsyncHandle *handle,
 
 			first_dir = data->dirs;
 			data->dirs = g_list_remove_link (data->dirs, first_dir);
-			dir = (gchar*) first_dir->data; 
+			dir = (char*) first_dir->data; 
 			g_list_free (first_dir);
 
-			good_dir_to_search_into = ! file_is_hidden (file_name_from_path (dir));
+			good_dir_to_search_into = ! cache_dir (file_name_from_path (dir));
 			if (good_dir_to_search_into)
 				search_dir_async (data, dir);
 			g_free (dir);
 		} while (! good_dir_to_search_into);
 
 	} else if (result != GNOME_VFS_OK) {
-		gchar *path;
+		char *path;
 
 		path = gnome_vfs_uri_to_string (data->uri, 
 						GNOME_VFS_URI_HIDE_NONE);
-		g_warning ("Cannot load directory \"%s\" : %s\n", path,
+		g_warning ("Cannot load directory \"%s\": %s\n", path,
 			   gnome_vfs_result_to_string (result));
 		g_free (path);
 
@@ -1210,13 +1282,24 @@ static void
 search_dir_async (DialogData *data, char *dir)
 {
 	char *escaped;
+	char *start_from;
+
+	if (data->search_comments) 
+		start_from = comments_get_comment_dir (dir);
+	else
+		start_from = g_strdup (dir);
 
 	_gtk_entry_set_locale_text (GTK_ENTRY (data->p_current_dir_entry), dir);
 
+	/**/
+
 	if (data->uri != NULL)
 		gnome_vfs_uri_unref (data->uri);
-	escaped = gnome_vfs_escape_path_string (dir);
+
+	escaped = gnome_vfs_escape_path_string (start_from);
 	data->uri = gnome_vfs_uri_new (escaped);
+
+	g_free (start_from);
 	g_free (escaped);
 
 	gnome_vfs_async_load_directory_uri (
@@ -1230,10 +1313,28 @@ search_dir_async (DialogData *data, char *dir)
 }
 
 
+static gboolean
+empty_pattern (const char *pattern)
+{
+	return (pattern == NULL) || (pattern[0] == 0);
+}
+
+
 static void 
 search_images_async (DialogData *data)
 {
+	SearchData *search_data = data->search_data;
+
 	free_search_results_data (data);
 	gtk_list_store_clear (data->p_progress_tree_model);
-	search_dir_async (data, data->search_data->start_from);
+
+	/* if the search criteria include comment data just search in
+	 * the comments tree */
+
+	data->search_comments = ! (empty_pattern (search_data->comment_pattern)
+				   && empty_pattern (search_data->place_pattern)
+				   && empty_pattern (search_data->keywords_pattern)
+				   && (search_data->date_scope == DATE_ANY));
+	
+	search_dir_async (data, search_data->start_from);
 }
